'use strict';

var _chai = require('chai');

var _chai2 = _interopRequireDefault(_chai);

var _spec_helper = require('./spec_helper.js');

var _spec_helper2 = _interopRequireDefault(_spec_helper);

var _morphological_utility = require('./../lib/morphological_utility.js');

var _morphological_utility2 = _interopRequireDefault(_morphological_utility);

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

var should = _chai2.default.should();
var expect = _chai2.default.expect;
var ShouldTransform = _spec_helper2.default.ShouldTransform;
var ShouldNotTransform = _spec_helper2.default.ShouldNotTransform;

describe("Stemmer", function () {
	it("should available", function () {
		expect(_morphological_utility2.default).to.exist;
	});

	describe("Total Syllables", function () {
		it("'memasak' should return 3", function () {
			_morphological_utility2.default.totalSyllables('memasak').should.equal(3);
		});

		it("'mewarnai' should return 4", function () {
			_morphological_utility2.default.totalSyllables('mewarnai').should.equal(4);
		});

		it("'permainan' should return 4", function () {
			_morphological_utility2.default.totalSyllables('permainan').should.equal(4);
		});
	});

	describe("Remove particle", function () {
		describe("should remove these particles at the end of the word", function () {
			it("'kah'", function () {
				ShouldTransform(_morphological_utility2.default.removeParticle, 'manakah', 'mana');
				ShouldTransform(_morphological_utility2.default.removeParticle, 'bukukah', 'buku');
			});

			it("'lah'", function () {
				ShouldTransform(_morphological_utility2.default.removeParticle, 'itulah', 'itu');
			});

			it("'pun'", function () {
				ShouldTransform(_morphological_utility2.default.removeParticle, 'bagaimanapun', 'bagaimana');
			});
		});
	});

	describe("should not remove these particles at the rest part of the word", function () {
		it("'kah'", function () {
			ShouldNotTransform(_morphological_utility2.default.removeParticle, 'kahak');
			ShouldNotTransform(_morphological_utility2.default.removeParticle, 'pernikahan');
		});

		it("'lah'", function () {
			ShouldNotTransform(_morphological_utility2.default.removeParticle, 'lahiriah');
			ShouldNotTransform(_morphological_utility2.default.removeParticle, 'kelahiran');
		});

		it("'pun'", function () {
			ShouldNotTransform(_morphological_utility2.default.removeParticle, 'punya');
			ShouldNotTransform(_morphological_utility2.default.removeParticle, 'kepunyaan');
		});
	});

	describe("Remove possesive pronoun", function () {
		describe("should remove these possessive pronouns at the end of the word", function () {
			it("'ku'", function () {
				ShouldTransform(_morphological_utility2.default.removePossesive, 'mainanku', 'mainan');
				ShouldTransform(_morphological_utility2.default.removePossesive, 'bukuku', 'buku');
			});

			it("'mu'", function () {
				ShouldTransform(_morphological_utility2.default.removePossesive, 'gelasmu', 'gelas');
			});

			it("'nya'", function () {
				ShouldTransform(_morphological_utility2.default.removePossesive, 'mobilnya', 'mobil');
			});
		});

		describe("should not remove these possessive pronouns at the rest part of the word", function () {
			it("'ku'", function () {
				ShouldNotTransform(_morphological_utility2.default.removePossesive, 'kumakan');
				ShouldNotTransform(_morphological_utility2.default.removePossesive, 'kekurangan');
			});

			it("'mu'", function () {
				ShouldNotTransform(_morphological_utility2.default.removePossesive, 'murahan');
				ShouldNotTransform(_morphological_utility2.default.removePossesive, 'kemurkaan');
			});

			it("'nya'", function () {
				ShouldNotTransform(_morphological_utility2.default.removePossesive, 'nyapu');
				ShouldNotTransform(_morphological_utility2.default.removePossesive, 'menyambung');
			});
		});
	});

	describe("Remove first order prefix", function () {
		describe("words with these special characters", function () {
			describe("at the begining", function () {
				describe("followed by a vowel, should remove and substitute the last character", function () {
					it("'meny'", function () {
						ShouldTransform(_morphological_utility2.default.removeFirstOrderPrefix, 'menyapu', 'sapu');
						ShouldTransform(_morphological_utility2.default.removeFirstOrderPrefix, 'menyanyi', 'nyanyi');
						ShouldTransform(_morphological_utility2.default.removeFirstOrderPrefix, 'menyala', 'nyala');
						ShouldTransform(_morphological_utility2.default.removeFirstOrderPrefix, 'menyata', 'nyata');
					});

					it("'pen'", function () {
						ShouldTransform(_morphological_utility2.default.removeFirstOrderPrefix, 'penulis', 'tulis');
					});

					it("'peny'", function () {
						ShouldTransform(_morphological_utility2.default.removeFirstOrderPrefix, 'penyongsong', 'songsong');
						ShouldTransform(_morphological_utility2.default.removeFirstOrderPrefix, 'penyakit', 'sakit');
						ShouldTransform(_morphological_utility2.default.removeFirstOrderPrefix, 'penyanyi', 'nyanyi');
					});
				});

				describe("followed by consonant, should only remove the special characters", function () {
					it("'meny'", function () {
						// Find a real indonesian word for this case
						ShouldTransform(_morphological_utility2.default.removeFirstOrderPrefix, 'menyxxx', 'xxx');
					});

					it("'peny'", function () {
						// Find a real indonesian word for this case
						ShouldTransform(_morphological_utility2.default.removeFirstOrderPrefix, 'penyxxx', 'xxx');
					});

					it("'pen'", function () {
						ShouldTransform(_morphological_utility2.default.removeFirstOrderPrefix, 'penjahat', 'jahat');
					});
				});
			});

			describe("at the rest part of the word", function () {
				describe("followed by a vowel, should not do anything", function () {
					it("'meny'", function () {
						ShouldNotTransform(_morphological_utility2.default.removeFirstOrderPrefix, 'xxxmenyaxx');
						ShouldNotTransform(_morphological_utility2.default.removeFirstOrderPrefix, 'xxxmenya');
					});

					it("'peny'", function () {
						ShouldNotTransform(_morphological_utility2.default.removeFirstOrderPrefix, 'xxxpenyaxx');
						ShouldNotTransform(_morphological_utility2.default.removeFirstOrderPrefix, 'xxxpenya');
					});

					it("'pen'", function () {
						ShouldNotTransform(_morphological_utility2.default.removeFirstOrderPrefix, 'xxxpenexx');
						ShouldNotTransform(_morphological_utility2.default.removeFirstOrderPrefix, 'xxxpeno');
					});
				});
			});

			describe("followed by consonant, should not do anything", function () {
				it("'meny'", function () {
					ShouldNotTransform(_morphological_utility2.default.removeFirstOrderPrefix, 'xxxmenykxx');
					ShouldNotTransform(_morphological_utility2.default.removeFirstOrderPrefix, 'xxxmenyk');
				});

				it("'peny'", function () {
					ShouldNotTransform(_morphological_utility2.default.removeFirstOrderPrefix, 'xxxpenykxx');
					ShouldNotTransform(_morphological_utility2.default.removeFirstOrderPrefix, 'xxxpenyk');
				});

				it("'pen'", function () {
					ShouldNotTransform(_morphological_utility2.default.removeFirstOrderPrefix, 'xxxpenrxx');
					ShouldNotTransform(_morphological_utility2.default.removeFirstOrderPrefix, 'xxxpenr');
				});
			});
		});

		describe("words with first order prefix characters", function () {
			describe("at the begining", function () {
				describe("should remove these characters", function () {
					it("'meng'", function () {
						ShouldTransform(_morphological_utility2.default.removeFirstOrderPrefix, 'menggambar', 'gambar');
					});

					it("'men'", function () {
						ShouldTransform(_morphological_utility2.default.removeFirstOrderPrefix, 'mendaftar', 'daftar');
					});

					it("'mem'", function () {
						ShouldTransform(_morphological_utility2.default.removeFirstOrderPrefix, 'membangun', 'bangun');
						ShouldTransform(_morphological_utility2.default.removeFirstOrderPrefix, 'membeli', 'beli');
						ShouldTransform(_morphological_utility2.default.removeFirstOrderPrefix, 'memberi', 'beri');
						ShouldTransform(_morphological_utility2.default.removeFirstOrderPrefix, 'memilah', 'pilah');
					});

					it("'me'", function () {
						ShouldTransform(_morphological_utility2.default.removeFirstOrderPrefix, 'melukis', 'lukis');
						ShouldTransform(_morphological_utility2.default.removeFirstOrderPrefix, 'memakan', 'makan');
						ShouldTransform(_morphological_utility2.default.removeFirstOrderPrefix, 'meminum', 'minum');
						ShouldTransform(_morphological_utility2.default.removeFirstOrderPrefix, 'menikah', 'nikah');
					});

					it("'peng'", function () {
						ShouldTransform(_morphological_utility2.default.removeFirstOrderPrefix, 'penggaris', 'garis');
					});

					it("'pem'", function () {
						ShouldTransform(_morphological_utility2.default.removeFirstOrderPrefix, 'pembajak', 'bajak');
					});

					it("'di'", function () {
						ShouldTransform(_morphological_utility2.default.removeFirstOrderPrefix, 'disayang', 'sayang');
					});

					it("'ter'", function () {
						ShouldTransform(_morphological_utility2.default.removeFirstOrderPrefix, 'terakhir', 'akhir');
						ShouldTransform(_morphological_utility2.default.removeFirstOrderPrefix, 'terucap', 'ucap');
					});

					it("'ke'", function () {
						ShouldTransform(_morphological_utility2.default.removeFirstOrderPrefix, 'kemana', 'mana');
						ShouldTransform(_morphological_utility2.default.removeFirstOrderPrefix, 'kemakan', 'makan');
					});
				});
			});

			describe("at the rest part of the word, should not remove these characters", function () {
				it("'meng'", function () {
					ShouldNotTransform(_morphological_utility2.default.removeFirstOrderPrefix, 'xxxmengxex');
					ShouldNotTransform(_morphological_utility2.default.removeFirstOrderPrefix, 'xexmeng');
				});

				it("'men'", function () {
					ShouldNotTransform(_morphological_utility2.default.removeFirstOrderPrefix, 'xxxmenxxx');
					ShouldNotTransform(_morphological_utility2.default.removeFirstOrderPrefix, 'xxxmen');
				});

				it("'mem'", function () {
					ShouldNotTransform(_morphological_utility2.default.removeFirstOrderPrefix, 'xxxmemxxx');
					ShouldNotTransform(_morphological_utility2.default.removeFirstOrderPrefix, 'xxxmem');
				});

				it("'me'", function () {
					ShouldNotTransform(_morphological_utility2.default.removeFirstOrderPrefix, 'xxxmexxx');
					ShouldNotTransform(_morphological_utility2.default.removeFirstOrderPrefix, 'xxxme');
				});

				it("'peng'", function () {
					ShouldNotTransform(_morphological_utility2.default.removeFirstOrderPrefix, 'xxxpengxxx');
					ShouldNotTransform(_morphological_utility2.default.removeFirstOrderPrefix, 'xxxpeng');
				});

				it("'pem'", function () {
					ShouldNotTransform(_morphological_utility2.default.removeFirstOrderPrefix, 'xxxpemxxx');
					ShouldNotTransform(_morphological_utility2.default.removeFirstOrderPrefix, 'xxxpem');
				});

				it("'di'", function () {
					ShouldNotTransform(_morphological_utility2.default.removeFirstOrderPrefix, 'xxxdixxx');
					ShouldNotTransform(_morphological_utility2.default.removeFirstOrderPrefix, 'xxxdi');
				});

				it("'ter'", function () {
					ShouldNotTransform(_morphological_utility2.default.removeFirstOrderPrefix, 'xxxterxxx');
					ShouldNotTransform(_morphological_utility2.default.removeFirstOrderPrefix, 'xxxter');
				});

				it("'ke'", function () {
					ShouldNotTransform(_morphological_utility2.default.removeFirstOrderPrefix, 'xxxkexxx');
					ShouldNotTransform(_morphological_utility2.default.removeFirstOrderPrefix, 'xxxke');
				});
			});
		});
	});

	describe("Remove Second Order Prefix", function () {
		describe("should handle these irregular words", function () {
			it("'belajar'", function () {
				ShouldTransform(_morphological_utility2.default.removeSecondOrderPrefix, 'belajar', 'ajar');
			});

			it("'belunjur'", function () {
				ShouldTransform(_morphological_utility2.default.removeSecondOrderPrefix, 'belunjur', 'unjur');
			});

			it("'pelajar'", function () {
				ShouldTransform(_morphological_utility2.default.removeSecondOrderPrefix, 'pelajar', 'ajar');
			});
		});

		describe("should handle words starting with 'be*er' where * isn't a vowel & the length > 4", function () {
			it("'beserta'", function () {
				ShouldTransform(_morphological_utility2.default.removeSecondOrderPrefix, 'beserta', 'serta');
			});
		});

		describe("words with second order prefix characters", function () {
			describe("at the beginning", function () {
				describe("should remove these characters", function () {
					it("'ber'", function () {
						ShouldTransform(_morphological_utility2.default.removeSecondOrderPrefix, 'bercerita', 'cerita');
					});

					it("'per'", function () {
						ShouldTransform(_morphological_utility2.default.removeSecondOrderPrefix, 'perjelas', 'jelas');
					});

					it("'pe'", function () {
						ShouldTransform(_morphological_utility2.default.removeSecondOrderPrefix, 'pesuruh', 'suruh');
					});
				});
			});
		});

		describe("at the rest part of the word, should not remove these characters", function () {
			it("'ber'", function () {
				ShouldNotTransform(_morphological_utility2.default.removeSecondOrderPrefix, 'xxxberxxx');
				ShouldNotTransform(_morphological_utility2.default.removeSecondOrderPrefix, 'xxxber');
			});

			it("'per'", function () {
				ShouldNotTransform(_morphological_utility2.default.removeSecondOrderPrefix, 'xxxperxxx');
				ShouldNotTransform(_morphological_utility2.default.removeSecondOrderPrefix, 'xxxper');
			});

			it("'pe'", function () {
				ShouldNotTransform(_morphological_utility2.default.removeSecondOrderPrefix, 'xxxpexxx');
				ShouldNotTransform(_morphological_utility2.default.removeSecondOrderPrefix, 'xxxpe');
			});
		});
	});

	describe("Remove Suffix", function () {
		describe("word with these suffix characters", function () {
			describe("at the end of the word, should remove the suffix characters", function () {
				it("'kan'", function () {
					ShouldTransform(_morphological_utility2.default.removeSuffix, 'katakan', 'kata');
				});

				it("'an'", function () {
					ShouldTransform(_morphological_utility2.default.removeSuffix, 'sandaran', 'sandar');
					ShouldTransform(_morphological_utility2.default.removeSuffix, 'makanan', 'makan');
				});

				it("'i'", function () {
					ShouldTransform(_morphological_utility2.default.removeSuffix, 'tiduri', 'tidur');
				});
			});
		});

		describe("at the rest part of the word, should not remove the characters", function () {
			it("'kan'", function () {
				ShouldNotTransform(_morphological_utility2.default.removeSuffix, 'kanxxx');
				ShouldNotTransform(_morphological_utility2.default.removeSuffix, 'xxxkanxxx');
			});

			it("'an'", function () {
				ShouldNotTransform(_morphological_utility2.default.removeSuffix, 'anxxx');
				ShouldNotTransform(_morphological_utility2.default.removeSuffix, 'xxxanxxx');
			});

			it("'an'", function () {
				ShouldNotTransform(_morphological_utility2.default.removeSuffix, 'ixxx');
				ShouldNotTransform(_morphological_utility2.default.removeSuffix, 'xxxixxx');
			});
		});
	});
});